@@ -39,7 +39,9 @@
 #endif
 
 #include <xorg-server.h>
-#include "psmcomm.h"
+#include <unistd.h>
+#include <sys/ioctl.h>
+#include <sys/mouse.h>
 #include <errno.h>
 #include <string.h>
 #include "synproto.h"
@@ -55,7 +57,6 @@ struct SynapticsHwInfo {
     unsigned int capabilities;		    /* Capabilities */
     unsigned int ext_cap;		    /* Extended Capabilities */
     unsigned int identity;		    /* Identification */
-    Bool hasGuest;			    /* Has a guest mouse */
 };
 
 /*
@@ -78,7 +79,7 @@ psm_synaptics_identify(int fd, synapticshw_t *ident)
 #define PSM_LEVEL_NATIVE	2
 
 static Bool
-PSMQueryIsSynaptics(LocalDevicePtr local)
+PSMQueryIsSynaptics(InputInfoPtr pInfo)
 {
     int ret;
     int level = PSM_LEVEL_NATIVE;
@@ -88,14 +89,14 @@ PSMQueryIsSynaptics(LocalDevicePtr local)
      * Otherwise HWINFO will not return the right id
      * And we will need native mode anyway ...
      */
-    SYSCALL(ret = ioctl(local->fd, MOUSE_SETLEVEL, &level));
+    SYSCALL(ret = ioctl(pInfo->fd, MOUSE_SETLEVEL, &level));
     if (ret != 0) {
-	xf86Msg(X_ERROR, "%s Can't set native mode\n", local->name);
+	xf86Msg(X_ERROR, "%s Can't set native mode\n", pInfo->name);
 	return FALSE;
     }
-    SYSCALL(ret = ioctl(local->fd, MOUSE_GETHWINFO, &mhw));
+    SYSCALL(ret = ioctl(pInfo->fd, MOUSE_GETHWINFO, &mhw));
     if (ret != 0) {
-	xf86Msg(X_ERROR, "%s Can't get hardware info\n", local->name);
+	xf86Msg(X_ERROR, "%s Can't get hardware info\n", pInfo->name);
 	return FALSE;
     }
 
@@ -103,7 +104,7 @@ PSMQueryIsSynaptics(LocalDevicePtr local)
 	return TRUE;
     } else {
 	xf86Msg(X_ERROR, "%s Found no Synaptics, found Mouse model %d instead\n",
-		local->name, mhw.model);
+		pInfo->name, mhw.model);
 	return FALSE;
     }
 }
@@ -133,49 +134,43 @@ convert_hw_info(const synapticshw_t *psm_ident, struct SynapticsHwInfo *synhw)
 }
 
 static Bool
-PSMQueryHardware(LocalDevicePtr local)
+PSMQueryHardware(InputInfoPtr pInfo)
 {
     synapticshw_t psm_ident;
     struct SynapticsHwInfo *synhw;
     SynapticsPrivate *priv;
 
-    priv = (SynapticsPrivate *)local->private;
+    priv = (SynapticsPrivate *)pInfo->private;
 
     if(!priv->proto_data)
-        priv->proto_data = xcalloc(1, sizeof(struct SynapticsHwInfo));
+        priv->proto_data = calloc(1, sizeof(struct SynapticsHwInfo));
     synhw = (struct SynapticsHwInfo*)priv->proto_data;
 
     /* is the synaptics touchpad active? */
-    if (!PSMQueryIsSynaptics(local))
+    if (!PSMQueryIsSynaptics(pInfo))
 	return FALSE;
 
-    xf86Msg(X_PROBED, "%s synaptics touchpad found\n", local->name);
+    xf86Msg(X_PROBED, "%s synaptics touchpad found\n", pInfo->name);
 
-    if (!psm_synaptics_identify(local->fd, &psm_ident))
+    if (!psm_synaptics_identify(pInfo->fd, &psm_ident))
 	return FALSE;
 
     convert_hw_info(&psm_ident, synhw);
 
-    /* Check to see if the host mouse supports a guest */
-    synhw->hasGuest = FALSE;
-    if (psm_ident.capPassthrough) {
-        synhw->hasGuest = TRUE;
-    }
-
     ps2_print_ident(synhw);
 
     return TRUE;
 }
 
 static Bool
-PSMReadHwState(LocalDevicePtr local,
+PSMReadHwState(InputInfoPtr pInfo,
 	       struct SynapticsProtocolOperations *proto_ops,
 	       struct CommData *comm, struct SynapticsHwState *hwRet)
 {
-    return psaux_proto_operations.ReadHwState(local, proto_ops, comm, hwRet);
+    return psaux_proto_operations.ReadHwState(pInfo, proto_ops, comm, hwRet);
 }
 
-static Bool PSMAutoDevProbe(LocalDevicePtr local)
+static Bool PSMAutoDevProbe(InputInfoPtr pInfo)
 {
     return FALSE;
 }
@@ -186,5 +181,5 @@ struct SynapticsProtocolOperations psm_proto_operations = {
     PSMQueryHardware,
     PSMReadHwState,
     PSMAutoDevProbe,
-    NULL /* ReadDevDimensions */
+    SynapticsDefaultDimensions
 };
